import java.io.Console;
import java.io.*;
import java.lang.String;
import java.util.*;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;


public class Kmedoids
{


	int amountOfClusters;
	photo[] totalPhotos;
	ArrayList<ArrayList<photo>> clusters;
	ArrayList<Integer> medoids;
	ArrayList<Integer> newMedoids;


	String inputFile = "10023 Photos";
	int howManyPhotos = 10023;


	public Kmedoids()

	{
		///GET INITIAL AMOUNT OF CLUSTERS

		String n = System.console().readLine("Enter input:");
		amountOfClusters = Integer.parseInt(n);

		///READ IN LIST OF PHOTO DATA

		totalPhotos = readSample();

		for (photo p : totalPhotos)
		{
		System.out.println(p.getLat() + " " + p.getLong() + " " + Arrays.toString(p.getTags()));
		}

		///CREATE INITIAL MEDOIDS

		medoids = generateMed(amountOfClusters);

		///ASSIGN EACH PHOTO TO A MEDOID
		///RECALCULATE MEDOIDS n TIMES

		int maxReCalc = 10;

		for(int i = 0;i<maxReCalc;i++)
		{
			clusters = createClusters(totalPhotos, medoids);
			medoids = recalcMedoids(clusters, totalPhotos);
		}

		///OUTPUT CLUSTERS

		outputClusters(clusters);
	}




	public static void main(String[] args)
	{
	new Kmedoids();
	}





///INPUT FILE

	public photo[] readSample()
	{
		///INITIALISE ARRAY FOR AMOUNT OF LINES IN SAMPLE
		int amountofPhotos = howManyPhotos; //HOW MANY PHOTOS IN THIS SAMPLE
		photo[] totalPhotos = new photo[amountofPhotos];

		for(int i=0;i<amountofPhotos;i++)
		{
			totalPhotos[i] = new photo();
		}

		File file = new File("E:/Samples/" + inputFile + ".txt");
        BufferedReader reader = null;

		try
		{
            reader = new BufferedReader(new FileReader(file));
            int count=0;
            String currentLine;
            //repeat until all lines are read
            while ((currentLine = reader.readLine()) != null)
            {
				///ASSIGN EACH LINE AS A NEW PHOTO CLASS
				String[] sSplit = currentLine.split("\t");
				double lat = Double.parseDouble(sSplit[0]);
				double longt = Double.parseDouble(sSplit[1]);
				String[] tags = sSplit[2].split(" ");
				totalPhotos[count] = new photo(lat,longt,count,tags);
				count++;
            }
        }

        catch (FileNotFoundException e)
        {
            e.printStackTrace();
        }

        catch (IOException e)
        {
            e.printStackTrace();
        }
        finally
        {
            try
            {
                if (reader != null)
                {
                    reader.close();
                }
            }

            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        return totalPhotos;
	}



	public ArrayList<Integer> generateMed(int totalMeds)
	{
		//GENERATE RANDOM NUMBERS AND STORE THOSE PHOTO INDEXES
		ArrayList<Integer> medoids = new ArrayList<Integer>();
		while(medoids.size()<totalMeds)
		{
			int random = 0 + (int)(Math.random()*totalPhotos.length);
			while(!medoids.contains(random))
			{
				medoids.add(random);
				System.out.println("added number " + random);
			}
		}
		//SOME CONSOLE OUTPUT
		String listString = "";
		for (int i = 0;i<medoids.size();i++)
		{
		    listString += (Integer.toString(medoids.get(i))) + "\t";
		}
		System.out.println(listString);

		return medoids;
	}


	public ArrayList<ArrayList<photo>> createClusters(photo[] totalPhotos, ArrayList<Integer> medoids)
	{
		ArrayList<ArrayList<photo>> clusters = new ArrayList<ArrayList<photo>>();
		for (int med : medoids)
		{
			ArrayList<photo> currentCluster = new ArrayList<photo>();
			for(photo p : totalPhotos)
			{
				double dist = 10000.00;
				int nearestMedoid = 0;
				for(int m : medoids)
				{
					double d = Math.sqrt(  (totalPhotos[m].getLat() - p.getLat()) * (totalPhotos[m].getLat() - p.getLat())  +   (totalPhotos[m].getLong() - p.getLong())  *  (totalPhotos[m].getLong() - p.getLong()) );
					if (d < dist)
					{
						dist = d;
						nearestMedoid=m;
					}
				}
				if (nearestMedoid == med)
				{
					currentCluster.add(p);
				}
			}
			clusters.add(currentCluster);
		}

		//SOME CONSOLE OUTPUT
		for (ArrayList<photo> cluster : clusters)
		{
			System.out.println("Cluster");
			for (photo p : cluster)
			{
				System.out.println(p.getLat() + " " + p.getLong() + " " + Arrays.toString(p.getTags()) + " " + p.getLoc());
			}
		}
		System.out.println("Amount of Clusters: " + clusters.size());
		for(int i = 0;i < clusters.size();i++)
		{
			System.out.println("Cluster " + i + " has " + clusters.get(i).size() + " photos");
		}

		return clusters;
	}


	public ArrayList<Integer> recalcMedoids(ArrayList<ArrayList<photo>> clusters, photo[] totalPhotos)
	{
		double newM;
		photo newMedoid;
		ArrayList<Integer> newMedoids = new ArrayList<Integer>();
		for (ArrayList<photo> cluster : clusters)
		{
			Map<photo, Double> LocAndDist = new Hashtable<photo, Double>();
			int x = 0;
			for (photo p : cluster)
			{
				double distance = 00.00;
				for (int i = 0;i<cluster.size();i++)
				{
					distance += Math.sqrt(  (p.getLat() - cluster.get(i).getLat()) * (p.getLat() - cluster.get(i).getLat())  +   (p.getLong() - cluster.get(i).getLong())  *  (p.getLong() - cluster.get(i).getLong()) );
				}
				LocAndDist.put(p,distance);
				x++;
			}
			Double smallestDist = 10000.00;
			photo p = new photo();
			for (Map.Entry<photo,Double> entry : LocAndDist.entrySet())
			{
				Double v = entry.getValue();
				if (v < smallestDist)
				{
				smallestDist = v;
				p = entry.getKey();
				}
			}
			newMedoids.add(p.getLoc());
		}
		for (int p : newMedoids)
		{
			System.out.println("new medoid " + p);
		}
		return newMedoids;
	}


	public void outputClusters(ArrayList<ArrayList<photo>> clusters)
	{
		int clusterNo = 0;
		for (ArrayList<photo> cluster : clusters)
		{
			try
			{
				//Create Directory
				File dir = new File("E:/Samples/ClustersFor" + inputFile);
				dir.mkdir();
				// Create file
				FileWriter fstream = new FileWriter("E:/Samples/ClustersFor" + inputFile + "/" + "Cluster Amount" + clusters.size() + "cluster" + clusterNo + ".tra",true);
				BufferedWriter out = new BufferedWriter(fstream);
				int count = 0;
				for (photo p : cluster)
				{
					out.write("\t" + p.getLat() + "\t" + p.getLong());
					for (String tag : p.getTags())
					{
						System.out.println("Outputting Tags");
						out.write("," + tag);
						//Close the output stream
					}
					out.write("\r\n");
				}
				out.close();
			}
			catch (Exception e)
			{
			//Catch exception if any
			System.err.println("Error: " + e.getMessage());
			}
			clusterNo++;
		}
	}
}





//PHOTO CLASS


class photo
{
	private double latitude;
	private double longitude;
	private int locInSample;
	private String[] tags;

	public photo(double lat, double longt, int count, String[] inputTags)
	{
		this.latitude = lat;
		this.longitude = longt;
		this.locInSample = count;
		this.tags = new String[inputTags.length];

		for(int i = 0;i<tags.length;i++)
		{
			tags[i] = inputTags[i];
		}

	}

	public photo()
	{
	}

	public double getLat()
	{
		return this.latitude;
	}

	public double getLong()
	{
		return this.longitude;
	}

	public int getLoc()
	{
		return this.locInSample;
	}

	public String[] getTags()
	{
		return this.tags;
	}


}